% we want to find a_k, b_k, k = 1,...,K of the convex PWA function
% f(x) := max a_k*x+b_k such that
%   \forall x: Jlow(x) <= max a_k*x+b_k <= Jup(k)
%   "K" is minimized (as a heurstics we minimize ||a_k||_1, ||b_k||_1)
% since Jlow() and Jup() are PWA functions, the first constraint reads
%   \forall i, x \in R_i: Clow_i*x+Dlow_i <= max a_k*x+b_k <= Cup_i*x+Dup_i
%
% in our case, Jlow(x) = V(x), where V(x) is the optimal cost function of a
% given explicit MPC solution (pretend it's a Lyapunov function for now),
% and Jup(x) = Jlow(x) + ||Qx||
%
% Jlow() and Jup() are convex PWA functions:
%   Jlow(x) := Clow_i*x+Dlow_i, if x \in R_i
%    Jup(x) := Cup_i*x+Dup_i, if x \in R_i

% to construct epigraphs, we need to find the maximum of a PWA function
% over its domain. for that, we need vertices of the domain
X = union(ctrl.Pfinal); VX = extreme(X);

% we are going to approximate the cost of a given explicit MPC solution
Clow = ctrl.Bi; for i = 1:length(Clow), Clow{i} = Clow{i}'; end
Dlow = ctrl.Ci;
R = ctrl.Pn;
V = pelemfun(@extreme, R);
nR = length(R);

% get Jup(x) = Jlow(x) + ||Qx||
[Cup, Dup] = J_add_norm(Clow, Dlow, R, ctrl.probStruct.Q);

% obtain polytopic representation of the epigraphs:
% * PJ is the epigraph of Jlow(x)
% * PJN is the epigraph of Jup(x)
% * Jpoly is the polytopic representation of the difference of the two
%   functions
[PJ, PJN, Jpoly, Jmax] = get_J_epigraph(R, X, Clow, Dlow, Cup, Dup);

close all
plot(Jpoly, 'y');

Jlow.R = R;
Jlow.C = Clow;
Jlow.D = Dlow;
Jup.R = R;
Jup.C = Cup;
Jup.D = Dup;
